/**
 * \file gui/export/export_masschroq_dialog/exportmasschroqdialog.cpp
 * \date 25/01/2019
 * \author Olivier Langella
 * \brief choose ODS export options
 */

/*******************************************************************************
 * Copyright (c) 2019 Olivier Langella <olivier.langella@u-psud.fr>.
 *
 * This file is part of XTPcpp.
 *
 *     XTPcpp is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTPcpp is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTPcpp.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Olivier Langella <olivier.langella@u-psud.fr> - initial API and
 *implementation
 ******************************************************************************/

#include "exportmasschroqdialog.h"

#include "ui_export_masschroq_dialog.h"
#include "exportmasschroqdialog.h"
#include <QDebug>
#include <QSettings>
#include <QMessageBox>
#include <QFileDialog>

ExportMasschroqDialog::ExportMasschroqDialog(QWidget *parent)
  : QDialog(parent), ui(new Ui::ExportMasschroqDialog)
{
  qDebug() << __FILE__ << " " << __FUNCTION__ << " " << __LINE__;
  ui->setupUi(this);
  this->setModal(true);

  ui->directorySelectWidget->setHidden(true);
  ui->msrunFilepathOkLabel->setHidden(true);
  
  ui->warningDirectoryLabel->setStyleSheet("QLabel { color : red; }");

#if QT_VERSION >= 0x050000
  // Qt5 code
#else
  // Qt4 code

#endif

  qDebug() << __FILE__ << " " << __FUNCTION__ << " " << __LINE__;
}

ExportMasschroqDialog::~ExportMasschroqDialog()
{
  delete ui;
}

void
ExportMasschroqDialog::setProject(const Project *p_project)
{
  mp_project = p_project;
}

void
ExportMasschroqDialog::setMasschroqFileParameters(
  const MasschroqFileParameters &params)
{
  ui->outputFileEdit->setText(params.result_file_name);
  ui->outputFileFormatComboBox->setCurrentText("ODS");
  if(params.result_file_format == TableFileFormat::tsv)
    {
      ui->outputFileFormatComboBox->setCurrentText("TSV");
    }

  ui->comparFileGroupBox->setChecked(params.export_compar_file);

  ui->comparFileEdit->setText(params.compar_file_name);
  ui->comparFileFormatComboBox->setCurrentText("ODS");
  if(params.compar_file_format == TableFileFormat::tsv)
    {
      ui->comparFileFormatComboBox->setCurrentText("TSV");
    }


  ui->timeCorrectionGroupBox->setChecked(params.write_alignment_times);
  ui->timeCorrectionDirectoryEdit->setText(params.alignment_times_directory);

  ui->ms2TendencySpinBox->setValue(params.ms2_tendency_half_window);
  ui->ms2SmoothingSpinBox->setValue(params.ms2_smoothing_half_window);
  ui->ms1SmoothingSpinBox->setValue(params.ms1_smoothing_half_window);
}


MasschroqFileParameters
ExportMasschroqDialog::getMasschroqFileParameters() const
{
  MasschroqFileParameters params;

  params.result_file_name   = ui->outputFileEdit->text();
  params.result_file_format = TableFileFormat::ods;
  if(ui->outputFileFormatComboBox->currentText() == "TSV")
    {
      params.result_file_format = TableFileFormat::tsv;
    }

  params.export_compar_file = ui->comparFileGroupBox->isChecked();
  params.compar_file_name   = ui->comparFileEdit->text();
  params.compar_file_format = TableFileFormat::ods;
  if(ui->comparFileFormatComboBox->currentText() == "TSV")
    {
      params.compar_file_format = TableFileFormat::tsv;
    }


  params.write_alignment_times     = ui->timeCorrectionGroupBox->isChecked();
  params.alignment_times_directory = ui->timeCorrectionDirectoryEdit->text();

  params.ms2_tendency_half_window  = ui->ms2TendencySpinBox->value();
  params.ms2_smoothing_half_window = ui->ms2SmoothingSpinBox->value();
  params.ms1_smoothing_half_window = ui->ms1SmoothingSpinBox->value();
  return params;
}


void
ExportMasschroqDialog::accept()
{
  QString error_message;
  if(ui->outputFileEdit->text().isEmpty())
    {
      error_message = QString(tr("MassChroqML result file name must be set"));
    }
  if(ui->comparFileGroupBox->isChecked())
    {
      if(ui->comparFileEdit->text().isEmpty())
        {
          error_message =
            QString(tr("MassChroqML compar file name must be set"));
        }
    }

  if(ui->timeCorrectionGroupBox->isChecked())
    {
      if(ui->timeCorrectionDirectoryEdit->text().isEmpty())
        {
          error_message = QString(tr(
            "the directory name to write alignment corrections must be set"));
        }
    }

  if(error_message.isEmpty())
    {
      QDialog::accept();
    }
  else
    {
      QMessageBox msgBox;
      msgBox.setWindowTitle(tr("MassChroQ parameter problem"));
      msgBox.setText(error_message);
      msgBox.setIcon(QMessageBox::Critical);
      msgBox.exec();
    }
}


void
ExportMasschroqDialog::doCheckMsrunFilepath()
{

  ui->msrunFilepathOkLabel->setHidden(true);
  bool is_ok = true;
  for(IdentificationDataSourceSp p_ident_data_source :
      mp_project->getIdentificationDataSourceStore()
        .getIdentificationDataSourceList())
    {
      MsRunSp msrun_sp = p_ident_data_source->getMsRunSp();

      if(msrun_sp->findMsRunFile())
        {
          // msrun_sp->checkMsRunStatistics();
        }
      else
        {
          is_ok = false;
        }
      if(!is_ok)
        {
          ui->warningDirectoryLabel->setText(
            tr("%1 not found : Please choose the directory to look for this MS "
               "run file.")
              .arg(msrun_sp.get()->getFilename()));
          ui->directorySelectWidget->setHidden(false);
          break;
        }
    }
  if(is_ok)
    {
      ui->directorySelectWidget->setHidden(true);

      ui->msrunFilepathOkLabel->setText(tr("All MS run files found"));
      ui->msrunFilepathOkLabel->setHidden(false);
    }
}

void
ExportMasschroqDialog::doBrowseMsrunDirectory()
{

  QSettings settings;
  QString path     = settings.value("path/mzdatadir", "").toString();
  QString filename = QFileDialog::getExistingDirectory(
    this, tr("Choose directory to look for MS runs"), QString("%1").arg(path));

  if(!filename.isEmpty())
    {
      QDir parent(filename);
      // parent.cdUp();
      settings.setValue("path/mzdatadir", parent.canonicalPath());
      doCheckMsrunFilepath();
      qDebug() << __FILE__ << " " << __FUNCTION__ << " " << __LINE__ << " "
               << parent.absolutePath();
    }
}
